﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace EditorTestbed
{
    public enum MessageType
    {
        USER,
        PROTOCOL
    };
    [Serializable]
    public struct MessageHeader
    {
        public uint id;
        public int transaction;
        public uint size;
        public uint type;
    };
    [Serializable]
    public class Message
    {
        public MessageHeader header;
        public byte[] data = null;
    };
    class MessageQueue
    {
        public void Enqueue(Message msg)
        {
            lock (m_messages)
            {
                m_messages.Enqueue(msg);
            }
            m_enqueueEvent.Set();
        }
        public Message Dequeue()
        {
            lock (m_messages)
            {
                if (m_messages.Count == 0)
                    return null;
                return m_messages.Dequeue();
            }
        }
        public void WaitForItem()
        {
            bool empty = false;
            lock (m_messages)
            {
                empty = m_messages.Count == 0;
            }
            if (empty)
                m_enqueueEvent.WaitOne();
        }
        public void Signal()
        {
            m_enqueueEvent.Set();
        }
        Queue<Message> m_messages = new Queue<Message>();
        AutoResetEvent m_enqueueEvent = new AutoResetEvent(false);
    }
}
