﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace EditorTestbed
{
    public partial class MessageStack : ToolWindow
    {
        public MessageStack(Client client)
        {
            m_client = client;
            InitializeComponent();
            if (m_messages == null)
                m_messages = m_client.m_stampedMessages;
        }

        Client m_client = null;
        bool m_realtimeReplay = false;
        DateTime m_startTime;
        List<bool> m_alreadyReplayedList = new List<bool>();
        StampedMessages m_messages = null;

        public bool RealtimeReplay
        {
            get { return m_realtimeReplay; }
        }

        public void RefreshList(StampedMessages messages)
        {
            m_realtimeReplay = false;
            listBox1.Items.Clear();
            m_messages = messages;
            foreach (StampedMessage msg in messages.Messages)
            {
                listBox1.Items.Add(msg);
            }
        }

        void SendMessage(Message msg)
        {
            if (msg.header.id == (uint)MessageID.CHANGE_OBJECT_PROPERTIES)
            {
                Message_ObjectProperties msgData = (Message_ObjectProperties)Client.ByteArrayToStructure(
                    msg.data, typeof(Message_ObjectProperties));
                Entity ent = Testbed.ms_instance.m_world.FindEntity(msgData.id);
                ent.MeshColor = Color.FromArgb((int)msgData.color);
                ent.VitalityPoints = msgData.vitalityPoints;
            }
            m_client.Send(msg);
        }

        private void listBox1_DoubleClick(object sender, EventArgs e)
        {
            if (m_realtimeReplay || listBox1.SelectedIndex < 0)
                return;
            StampedMessage smsg = listBox1.Items[listBox1.SelectedIndex] as StampedMessage;
            if (smsg != null)
            {
                SendMessage(smsg.msg);
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (!m_realtimeReplay)
                return;
            TimeSpan timeOffset = DateTime.Now - m_startTime;
            for (int i = 0; i < listBox1.Items.Count; ++i)
            {
                if (m_alreadyReplayedList[i])
                    continue;
                StampedMessage smsg = listBox1.Items[i] as StampedMessage;
                if (timeOffset >= smsg.timeOffset)
                {
                    m_alreadyReplayedList[i] = true;
                    listBox1.SelectedIndices.Clear();
                    listBox1.SelectedIndex = i;
                    SendMessage(smsg.msg);
                    if (i == listBox1.Items.Count - 1)
                    {
                        comboBox1.Enabled = true;
                        m_realtimeReplay = false;
                    }
                }
            }
        }

        private void replayFromHereToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedIndex < 0)
                return;

            m_alreadyReplayedList.Clear();
            for (int i = 0; i < listBox1.Items.Count; ++i)
                m_alreadyReplayedList.Add(i < listBox1.SelectedIndex);
            m_startTime = DateTime.Now - ((StampedMessage)listBox1.SelectedItem).timeOffset;
            m_realtimeReplay = true;
            comboBox1.Enabled = false;
            timer1.Start();
        }

        void SaveMessages(StampedMessages smsgs)
        {
            string name = "";
            StampedMessages currentSmsgs = comboBox1.SelectedItem as StampedMessages;
            if (currentSmsgs != null)
                name = currentSmsgs.Name;
            EnterNameDialog enterName = new EnterNameDialog(name);
            if (enterName.ShowDialog() != DialogResult.OK)
                return;
            name = enterName.GetName();

            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                smsgs.Name = name;
                smsgs.Save(saveFileDialog1.FileName);
            }
        }

        private void saveSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (listBox1.SelectedItems.Count == 0)
                return;

            StampedMessages smsgs = new StampedMessages();
            foreach (Object obj in listBox1.SelectedItems)
            {
                StampedMessage msg = obj as StampedMessage;
                smsgs.Messages.Add(msg);
            }
            SaveMessages(smsgs);
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            StampedMessages smsgs = comboBox1.SelectedItem as StampedMessages;
            if (smsgs != null)
                RefreshList(smsgs);
        }

        private void loadMessagesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                StampedMessages smsgs = new StampedMessages();
                smsgs.Load(openFileDialog1.FileName);
                comboBox1.Items.Add(smsgs);
            }
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RefreshList(m_messages);
        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            if (m_realtimeReplay)
            {
                refreshToolStripMenuItem.Enabled = false;
                loadMessagesToolStripMenuItem.Enabled = false;
                saveSelectedToolStripMenuItem.Enabled = false;
                saveAllToolStripMenuItem.Enabled = false;
                replayFromHereToolStripMenuItem.Enabled = false;
            }
            else
            {
                refreshToolStripMenuItem.Enabled = true;
                saveAllToolStripMenuItem.Enabled = true;
                saveSelectedToolStripMenuItem.Enabled = (listBox1.SelectedItems.Count > 0);
                replayFromHereToolStripMenuItem.Enabled = (listBox1.SelectedItems.Count > 0);
            }
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < listBox1.Items.Count; ++i)
                listBox1.SelectedIndex = i;
        }

        private void saveAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StampedMessages smsgs = new StampedMessages();
            foreach (Object obj in listBox1.Items)
            {
                StampedMessage msg = obj as StampedMessage;
                smsgs.Messages.Add(msg);
            }
            SaveMessages(smsgs);
        }
    }
}
