﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Reflection;

namespace EditorTestbed
{
    public enum MessageID
    {
        HANDSHAKE,
        CREATE_OBJECT,
        CHANGE_OBJECT_PROPERTIES,
        SELECT_OBJECT,
        DELETE_OBJECT,
        KEY_DOWN,
        KEY_UP,
        MOUSE_DOWN,
        MOUSE_UP,
        MOUSE_MOVE,
        VIEWPORT_RESIZE,
        CLEAR_WORLD,
        CRASH_GAME
    };
    public enum MouseButton
    {
        LEFT,
        MIDDLE,
        RIGHT
    };

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct Message_Handshake
    {
        public IntPtr hwnd;
        public int width;
        public int height;
        public uint nextID;
    };

    public enum ObjectType
    {
        CUBE,
        SPHERE,
        TORUS
    };
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct Message_CreateObject
    {
        public byte objectType;
        public uint id;
    };
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct Message_ObjectID
    {
        public uint id;
    };
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct Message_ObjectProperties
    {
        public uint id;
        public uint color;
        // Position
        public float x, y, z;
        public int vitalityPoints;
    };
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct Message_KeyboardEvent
    {
        public int key;
    };
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct Message_MouseEvent
    {
        public byte button;
    };
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct Message_MouseMoveEvent
    {
        public int x, y;
    };
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct Message_Viewport
    {
        public int width;
        public int height; 
    };

    public class MessageReflection
    {
        static string GetCPPName(string sharpName)
        {
            if (sharpName == "Int32")
                return "rde::int32_t";
            else if (sharpName == "UInt32")
                return "rde::uint32_t";
            else if (sharpName == "Byte")
                return "rde::uint8_t";
            else if (sharpName == "Single")
                return "float";
            else if (sharpName.IndexOf("IntPtr") >= 0)
                return "void*";
            else
                return "UNKNOWN";
        }
        public static void ReflectTypes(string fileName)
        {
            TextWriter tw = new StreamWriter(fileName);
            WriteHeader(tw);

            ReflectEnum(tw, "EditorTestbed.MessageID");
            ReflectEnum(tw, "EditorTestbed.MouseButton");
            ReflectEnum(tw, "EditorTestbed.ObjectType");

            tw.WriteLine("\n\n#pragma pack(push, 1)");
            ReflectType(tw, "EditorTestbed.Message_Handshake");
            ReflectType(tw, "EditorTestbed.Message_CreateObject");
            ReflectType(tw, "EditorTestbed.Message_ObjectID");
            ReflectType(tw, "EditorTestbed.Message_ObjectProperties");
            ReflectType(tw, "EditorTestbed.Message_KeyboardEvent");
            ReflectType(tw, "EditorTestbed.Message_MouseEvent");
            ReflectType(tw, "EditorTestbed.Message_MouseMoveEvent");
            ReflectType(tw, "EditorTestbed.Message_Viewport");
            tw.WriteLine("\n#pragma pack(pop)");
            tw.WriteLine("");
            tw.Close();
        }
        static void WriteHeader(TextWriter tw)
        {
            tw.WriteLine("#pragma once");
            tw.WriteLine("// MACHINE GENERATED, DO NOT EDIT\n");
            tw.WriteLine("#include \"core/Config.h\"\n");
        }
        static void ReflectEnum(TextWriter tw, string typeName)
        {
            Type t = Type.GetType(typeName);
            tw.WriteLine("namespace {0}", t.Name);
            tw.WriteLine("{");
            tw.WriteLine("\tenum {0}", t.Name);
            tw.WriteLine("\t{");
            MemberInfo[] members = t.GetMembers();
            foreach (MemberInfo mi in members)
            {
                if (mi.MemberType == MemberTypes.Field && mi.Name != "value__")
                    tw.WriteLine("\t\t{0},", mi.Name);
            }
            tw.WriteLine("\t};");
            tw.WriteLine("}");
        }
        static void ReflectType(TextWriter tw, string typeName)
        {
            Type t = Type.GetType(typeName);
            tw.WriteLine("struct {0}", t.Name);
            tw.WriteLine("{");
            FieldInfo[] fields = t.GetFields(BindingFlags.Instance | BindingFlags.Public);
            foreach (FieldInfo f in fields)
            {
                tw.WriteLine("\t{0}\t{1};", GetCPPName(f.FieldType.Name), f.Name);
            }
            tw.WriteLine("};");
        }
    }
}
 