﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace EditorTestbed
{
    [Serializable]
    public class StampedMessage
    {
        public StampedMessage(Message m, TimeSpan offset)
        {
            msg = m;
            timeOffset = offset;
        }
        string MessageToString()
        {
            MessageID id = (MessageID)msg.header.id;
            string[] objectTypes = { "CUBE", "SPHERE", "TORUS" };
            switch (id)
            {
                case MessageID.CREATE_OBJECT:
                    return "CreateObject " + objectTypes[msg.data[0]];
                case MessageID.CHANGE_OBJECT_PROPERTIES:
                    return "ChangeObjectProperties";
                case MessageID.KEY_DOWN:
                    return "Key down";
                case MessageID.KEY_UP:
                    return "Key up";
                case MessageID.MOUSE_DOWN:
                    return "Mouse down";
                case MessageID.MOUSE_UP:
                    return "Mouse up";
                case MessageID.MOUSE_MOVE:
                    return "Mouse move";
                case MessageID.CLEAR_WORLD:
                    return "Clear world";
                case MessageID.VIEWPORT_RESIZE:
                    return "Viewport resize";
                case MessageID.CRASH_GAME:
                    return "CrashGame";
            }
            return "<Unknown>";
        }
        public override string ToString()
        {
            return MessageToString() + " - " + timeOffset.TotalSeconds.ToString();
        }
        public Message msg;
        public TimeSpan timeOffset;
    };

    public class StampedMessages
    {
        const int kMaxMessages = 65536;

        public bool IsEmpty()
        {
            return m_messages.Count == 0;
        }
        public void AddMessage(Message msg)
        {
            // @fixme Rather ugly hack. We dont want to save handshakes.
            if (msg.header.id == (uint)MessageID.HANDSHAKE)
                return;

            if (m_messages.Count == 0)
                m_firstMessageTime = DateTime.Now;

            StampedMessage smsg = new StampedMessage(msg, DateTime.Now - m_firstMessageTime);
            m_messages.Add(smsg);
            if (m_messages.Count >= kMaxMessages)
                m_messages.RemoveAt(0);
        }

        public void Save(Stream s, BinaryFormatter formatter)
        {
            formatter.Serialize(s, m_name);
            //formatter.Serialize(s, m_firstMessageTime);
            formatter.Serialize(s, m_messages);
        }
        public void Save(string fileName)
        {
            Stream s = File.Open(fileName, FileMode.Create);
            BinaryFormatter formatter = new BinaryFormatter();
            Save(s, formatter);
            s.Close();
        }
        public void Load(Stream s, BinaryFormatter formatter)
        {
            m_name = (string)formatter.Deserialize(s);
            //m_firstMessageTime = (DateTime)formatter.Deserialize(s);
            m_messages = (List<StampedMessage>)formatter.Deserialize(s);
        }
        public void Load(string fileName)
        {
            Stream s = File.Open(fileName, FileMode.Open);
            BinaryFormatter formatter = new BinaryFormatter();
            Load(s, formatter);
            s.Close();
        }

        string m_name = "<No name>";
        DateTime m_firstMessageTime;
        List<StampedMessage> m_messages = new List<StampedMessage>();

        public override string ToString()
        {
 	        return Name;
        }

        public string Name
        {
            get { return m_name; }
            set { m_name = value; }
        }
        public List<StampedMessage> Messages
        {
            get { return m_messages; }
        }
    }
}
