#include "core/ThreadEvent.h"
#include "core/Console.h"
#include "core/RdeAssert.h"
#include "core/win32/Windows.h"

namespace rde
{
ThreadEvent::ThreadEvent(bool initialState)
{
	m_hEvent = ::CreateEvent(0, TRUE, initialState, 0);
	RDE_ASSERT(m_hEvent != 0);
	if (m_hEvent == 0)
		Console::Warningf("Couldn't create event\n");
}
ThreadEvent::~ThreadEvent()
{
	if (m_hEvent && m_hEvent != INVALID_HANDLE_VALUE)
	{
		const bool ok = (::CloseHandle(m_hEvent) != 0);
		RDE_ASSERT(ok);
	}
	m_hEvent = 0;
}
void ThreadEvent::Signal()
{
	RDE_ASSERT(m_hEvent != 0);
	const bool ok = ::SetEvent(m_hEvent) != 0;
	RDE_ASSERT(ok);
}
void ThreadEvent::Reset()
{
	RDE_ASSERT(m_hEvent != 0);
	const bool ok = ::ResetEvent(m_hEvent) != 0;
	RDE_ASSERT(ok);
}
void ThreadEvent::WaitInfinite() const
{
	RDE_ASSERT(m_hEvent != 0);
	const DWORD result = ::WaitForSingleObject(m_hEvent, INFINITE);
	RDE_ASSERT(result == WAIT_OBJECT_0);
}
bool ThreadEvent::WaitTimeout(long milliseconds) const
{
	RDE_ASSERT(m_hEvent != 0);
	const DWORD result = WaitForSingleObject(m_hEvent, milliseconds);
	RDE_ASSERT(result == WAIT_TIMEOUT || result == WAIT_OBJECT_0);
	return result == WAIT_OBJECT_0;
}
void* ThreadEvent::GetSystemRepresentation() const
{
	return m_hEvent;
}

}
