#include "ipc/MessageQueue.h"
#include "ipc/Message.h"
#include "core/LockGuard.h"
#include "core/RdeAssert.h"

namespace rde
{
MessageQueue::MessageQueue()
:	m_head(0),
	m_tail(0),
	m_numItems(0)
{
	/**/
}

void MessageQueue::Enqueue(Message* msg)
{
	m_mutex.Acquire();
	if (m_tail == 0)	// empty queue?
	{
		RDE_ASSERT(m_head == 0);
		RDE_ASSERT(m_numItems == 0);
		m_head = m_tail = msg;
	}
	else
	{
		m_tail->m_next = msg;
		msg->m_next = 0;
		m_tail = msg;
	}
	++m_numItems;
	m_mutex.Release();
	Signal();
}

Message* MessageQueue::Dequeue()
{
	Message* result(0);
	LockGuard<Mutex> lock(m_mutex);
	if (m_head != 0)
	{
		RDE_ASSERT(m_numItems > 0);
		result = m_head;
		m_head = m_head->m_next;
		if (m_head == 0)	// queue empty.
		{
			RDE_ASSERT(m_numItems == 1);
			m_tail = 0;
		}
		result->m_next = 0;
		--m_numItems;
	}
	else
	{
		RDE_ASSERT(m_numItems == 0);
	}
	return result;
}

void MessageQueue::Clear()
{
	LockGuard<Mutex> lock(m_mutex);
	Message* iter = m_head;
	while (iter)
	{
		Message* next = iter->m_next;
		delete iter;
		iter = next;
	}
	m_head = m_tail = 0;
	m_numItems = 0;
}

void MessageQueue::WaitForItem()
{
	if (m_numItems == 0) // should be atomic on x86
		m_enqueueEvent.WaitInfinite();
}
void MessageQueue::Signal()
{
	m_enqueueEvent.Signal();
}

}
