#pragma once
#ifndef IPC_PIPE_CONNECTION_H
#define IPC_PIPE_CONNECTION_H

#include "ipc/Pipe.h"
#include "ipc/MessageQueue.h"
#include "core/Mutex.h"
#include "core/Thread.h"
#include "core/ThreadEvent.h"

// Architecture from Nocturnal Framework by Insomniac.

namespace rde
{
class Message;

namespace ConnectionState
{
	enum Enum
	{
		WAITING,
		ACTIVE,
		CLOSED,
		FAILED
	};
}

namespace ProtocolMessage
{
	enum Enum
	{
		DISCONNECT
	};
}

class PipeConnection
{
public:
	PipeConnection();
	~PipeConnection();

	bool ServerInit(const char* name, const char* pipeName);
	void Close();

	Message* CreateMessage(uint32_t id, uint32_t size, uint32_t type);
	bool Send(Message*);
	// Caller becomes owner of message, needs to delete it when
	// done. May return NULL if no messages in the queue or not connected.
	// Waiting will block until message is available.
	bool Receive(Message**, bool wait = false);
	bool SendProtocolMessage(ProtocolMessage::Enum msgID);

	ConnectionState::Enum GetState() const	{ return m_state; }

private:
	void ServerThread();
	void ConnectThread();
	void ReadThread();
	bool Read(void* buffer, uint32_t bytes);
	bool ReadPump();
	bool ReadMessage(Message**);
	void HandleProtocolMessage(Message*);
	void WriteThread();
	bool WritePump();
	bool WriteMessage(const Message*);
	bool Write(const void* buffer, uint32_t bytes);
	void SetState(ConnectionState::Enum);
	
	static const size_t kMaxName = 256;
	char					m_name[kMaxName];
	char					m_pipeName[kMaxName];
	char					m_readName[kMaxName];
	char					m_writeName[kMaxName];
	Pipe					m_readPipe;
	Pipe					m_writePipe;
	volatile bool			m_terminating;
	Thread					m_connectThread;
	ThreadEvent				m_terminateEvent;
	ConnectionState::Enum	m_state;
	Mutex					m_stateMutex;
	Thread					m_readThread;
	Thread					m_writeThread;
	int32_t					m_nextTransaction;
	MessageQueue			m_readQueue;
	MessageQueue			m_writeQueue;
};
}

#endif
