#include "PreCompiled.h"
#include "Application.h"
#include "ComPtr.h"
#include "DX9Utils.h"
#include "Mesh.h"
#include "Scene.h"
#include "core/Console.h"

#pragma comment(lib, "d3d9.lib")
#pragma comment(lib, "d3dx9.lib")

struct Application::Impl
{
	bool Init(HWND hWindow, int width, int height)
	{
		m_d3d = rde::ComPtr<IDirect3D9>(Direct3DCreate9(D3D_SDK_VERSION), rde::DontAddRef);

		D3DPRESENT_PARAMETERS d3dpp;
		ZeroMemory(&d3dpp, sizeof(d3dpp));
		d3dpp.BackBufferWidth = width;
		d3dpp.BackBufferHeight = height;
		D3DDISPLAYMODE d3dMode;
		m_d3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3dMode);
		d3dpp.BackBufferFormat = d3dMode.Format;
		d3dpp.BackBufferCount = 1;
		d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
		d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
		d3dpp.EnableAutoDepthStencil = true;
		d3dpp.AutoDepthStencilFormat = D3DFMT_D24S8;
		d3dpp.Windowed = true;
		d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
		d3dpp.Flags = D3DPRESENTFLAG_DISCARD_DEPTHSTENCIL | D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;

		HRESULT hr;
		hr = m_d3d->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWindow, 
			D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, m_device.GetPointer());
		DX_CHECK_FATAL(hr);

		return true;
	}

	rde::ComPtr<IDirect3D9>	m_d3d;
	rde::ComPtr<IDirect3DDevice9> m_device;
};

Application::Application()
{
	m_impl = new Impl();
}
Application::~Application()
{
	delete m_impl;
	m_impl = 0;
}
	
bool Application::Init(HWND hWindow, int width, int height)
{
	return m_impl->Init(hWindow, width, height);
}

void Application::Frame(float, Scene& scene, Mesh* selectedMesh)
{
	if (SUCCEEDED(m_impl->m_device->BeginScene()))
	{
		m_impl->m_device->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 
			D3DXCOLOR(0.f, 0.f, 0.f, 1.f), 1.0f, 0);
		scene.Render();
		scene.RenderSelected(selectedMesh);
		m_impl->m_device->EndScene();
		m_impl->m_device->Present(NULL,NULL,NULL,NULL);
	}
}

Mesh* Application::CreateCube(float width, float height, float depth)
{
	Mesh* mesh = new Mesh();
	DX_CHECK(::D3DXCreateBox(m_impl->m_device.Get(), width, height, depth, 
		mesh->m_mesh.GetPointer(), 0));
	return mesh;
}
Mesh* Application::CreateSphere(float radius)
{
	Mesh* mesh = new Mesh();
	DX_CHECK(::D3DXCreateSphere(m_impl->m_device.Get(), radius, 20, 20,
		mesh->m_mesh.GetPointer(), 0));
	return mesh;
}
Mesh* Application::CreateTorus(float innerRadius, float outerRadius)
{
	Mesh* mesh = new Mesh();
	DX_CHECK(::D3DXCreateTorus(m_impl->m_device.Get(), innerRadius, outerRadius, 20, 20,
		mesh->m_mesh.GetPointer(), 0));
	return mesh;
}

rde::ComPtr<ID3DXEffect> Application::CreateEffect(const char* fxName)
{
	rde::ComPtr<ID3DXEffect> effect;
    LPD3DXBUFFER compilationErrors = 0;
	DX_CHECK(::D3DXCreateEffectFromFile(m_impl->m_device.Get(), fxName, 0, 0, 
		D3DXSHADER_DEBUG, 0, effect.GetPointer(), &compilationErrors));	
	if (compilationErrors)
	{
		rde::Console::Warningf("Shader error: %s\n", compilationErrors->GetBufferPointer());
		compilationErrors->Release();
	}
	return effect;
}