#include "PreCompiled.h"
#include "Camera.h"
#include "core/RdeAssert.h"

Camera::Viewport::Viewport()
:	x(0), y(0), width(0), height(0)
{
}
Camera::Viewport::Viewport(int x_, int y_, int w, int h)
:	x(x_), y(y_), width(w), height(h)
{
}

Camera::Camera()
:	m_horizontalFov(1.f),
	m_nearPlaneDist(1.f),
	m_farPlaneDist(1000.f),
	m_aspect(1.f),
	m_projectionDirty(true)
{
	/**/
}

void Camera::Setup(float nearPlaneDist, float farPlaneDist, float horzFov)
{
	RDE_ASSERT(nearPlaneDist > 0.f);
	RDE_ASSERT(nearPlaneDist < farPlaneDist);
	RDE_ASSERT(horzFov > 0.f);
	m_nearPlaneDist = nearPlaneDist;
	m_farPlaneDist = farPlaneDist;
	m_horizontalFov = horzFov;
	m_projectionDirty = true;
}

void Camera::SetViewport(int x, int y, int width, int height)
{
	RDE_ASSERT(width > 0 && height > 0);
	m_viewport = Viewport(x, y, width, height);
	m_aspect = float(width) / height;
	m_projectionDirty = true;
}

void Camera::UpdateTransforms()
{
	if (m_projectionDirty)
	{
		::D3DXMatrixPerspectiveFovLH(&m_projection, m_horizontalFov, m_aspect, 
			m_nearPlaneDist, m_farPlaneDist);
		m_projectionDirty = false;
	}
	::D3DXMatrixInverse(&m_view, 0, &GetWorldTransform());
	::D3DXMatrixMultiply(&m_viewProjection, &m_view, &m_projection);
}

