#pragma once

namespace rde
{
enum EDontAddRef { DontAddRef };
template<typename T>
class ComPtr
{
public:
	explicit ComPtr(T* ptr = 0)
	:	m_ptr(ptr)
	{
		AddRef();
	}
	ComPtr(T* ptr, EDontAddRef): m_ptr(ptr) {}
	ComPtr(const ComPtr& rhs): m_ptr(rhs.m_ptr) 
	{
		AddRef();
	}
	~ComPtr()
	{
		if (m_ptr)
		{
			int refsLeft = m_ptr->Release();
			int k = refsLeft;
			(void)k;
		}
		m_ptr = 0;
	}

	ComPtr& operator=(const ComPtr& rhs)
	{
		if (this != &rhs)
		{
			rhs.AddRef();
			if (m_ptr)
				m_ptr->Release();
			m_ptr = rhs.m_ptr;
		}
		return *this;
	}
	ComPtr& operator=(T* ptr)
	{
		if (m_ptr != ptr)
		{
			if (ptr)
				ptr->AddRef();
			if (m_ptr)
				m_ptr->Release();
			m_ptr = ptr;
		}
		return *this;
	}

	T* operator->()				{ return m_ptr; }
	T const* operator->() const	{ return m_ptr; }

	T* Get() const	{ return m_ptr; }
	T** GetPointer()	{ return &m_ptr; }

private:
	__forceinline void AddRef() const
	{
		if (m_ptr)
			m_ptr->AddRef();
	}

	T*	m_ptr;
};

}
