#include "PreCompiled.h"
#include "Node.h"
#include "core/RdeAssert.h"

Node::Node()
:	m_isWorldTransformDirty(true)
{
	m_position.x = m_position.y = m_position.z = 0.f;
	::D3DXQuaternionIdentity(&m_rotation);
}

void Node::Update()
{
	if (m_isWorldTransformDirty)
		UpdateWorldTransform();
}

void Node::SetPosition(const D3DXVECTOR3& pos)
{
	m_position = pos;
	m_isWorldTransformDirty = true;
}

const D3DXMATRIX& Node::GetWorldTransform() const
{
	if (m_isWorldTransformDirty)
		UpdateWorldTransform();
	return m_worldTransform;
}

void Node::UpdateWorldTransform() const
{
	RDE_ASSERT(m_isWorldTransformDirty);
	::D3DXMatrixAffineTransformation(&m_worldTransform, 1.f, NULL, &m_rotation, &m_position);
	OnWorldTransformChange(m_worldTransform);
	m_isWorldTransformDirty = true;
}

